@@ -1,5 +1,6 @@
 import os
 import requests
+import urllib.error
 from urllib import request, parse
 from pma_python import core, pma
 
@@ -101,11 +102,11 @@ def get_training_sessions_for_participant(pmacontrolURL, participantUsername, pm
 
 	return new_training_session_dict
 
-def get_training_session_participants(pmacontrolURL, pmacontrolSessionID, pmacoreSessionID):
+def get_training_session_participants(pmacontrolURL, pmacontrolTrainingSessionID, pmacoreSessionID):
 	"""
 	Extract the participants in a particular session
 	"""
-	url = pma._pma_join(pmacontrolURL, "api/Sessions/" + str(pmacontrolSessionID) + "/Participants?sessionID=" + pma._pma_q(pmacoreSessionID))
+	url = pma._pma_join(pmacontrolURL, "api/Sessions/" + str(pmacontrolTrainingSessionID) + "/Participants?sessionID=" + pma._pma_q(pmacoreSessionID))
 	try:
 		headers = {'Accept': 'application/json'}
 		r = pma._pma_http_get(url, headers)
@@ -117,16 +118,16 @@ def get_training_session_participants(pmacontrolURL, pmacontrolSessionID, pmacor
 		parts[part['User']] = part
 	return parts
 
-def is_participant_in_training_session(pmacontrolURL, participantUsername, pmacontrolSessionID, pmacoreSessionID):
+def is_participant_in_training_session(pmacontrolURL, participantUsername, pmacontrolTrainingSessionID, pmacoreSessionID):
 	"""
 	Check to see if a specific user participates in a specific session
 	"""
-	all_parts = get_training_session_participants(pmacontrolURL, pmacontrolSessionID, pmacoreSessionID)
+	all_parts = get_training_session_participants(pmacontrolURL, pmacontrolTrainingSessionID, pmacoreSessionID)
 	return participantUsername in all_parts.keys()
 
-def get_training_session_url(pmacontrolURL, participantSessionID, participantUsername, pmacontrolSessionID, pmacontrolCaseCollectionID, pmacoreSessionID):
-	if (is_participant_in_training_session(pmacontrolURL, participantUsername, pmacontrolSessionID, pmacoreSessionID)):
-		for k, v in get_training_session(pmacontrolURL, pmacontrolSessionID, pmacoreSessionID)["CaseCollections"].items():
+def get_training_session_url(pmacontrolURL, participantSessionID, participantUsername, pmacontrolTrainingSessionID, pmacontrolCaseCollectionID, pmacoreSessionID):
+	if (is_participant_in_training_session(pmacontrolURL, participantUsername, pmacontrolTrainingSessionID, pmacoreSessionID)):
+		for k, v in get_training_session(pmacontrolURL, pmacontrolTrainingSessionID, pmacoreSessionID)["CaseCollections"].items():
 			if k == pmacontrolCaseCollectionID:
 				return v["Url"] + "?SessionID=" + participantSessionID
 	else:
@@ -147,50 +148,78 @@ def get_all_participants(pmacontrolURL, pmacoreSessionID):
 
 	return user_dict
 
-def register_participant_for_training_session(pmacontrolURL, participantUsername, pmacontrolSessionID, pmacontrolRole, pmacoreSessionID):
+def register_participant_for_training_session(pmacontrolURL, participantUsername, pmacontrolTrainingSessionID, pmacontrolRole, pmacoreSessionID, pmacontrolInteractionMode = pma_interaction_mode_locked):
 	"""
 	Registers a particpant for a given session, assign a specific role
 	"""
-	#if is_participant_in_training_session(pmacontrolURL, participantUsername, pmacontrolSessionID, pmacoreSessionID):
-	#	raise NameError ("PMA.core user " + participantUsername + " is ALREADY registered in PMA.control training session " + str(pmacontrolSessionID))
-	url = pma._pma_join(pmacontrolURL, "api/Sessions/") + str(pmacontrolSessionID) + "/AddParticipant?SessionID=" + pmacoreSessionID
-	data = { "UserName": participantUsername, "Role": pmacontrolRole}   # default interaction mode = Locked
+	#if is_participant_in_training_session(pmacontrolURL, participantUsername, pmacontrolTrainingSessionID, pmacoreSessionID):
+	#	raise NameError ("PMA.core user " + participantUsername + " is ALREADY registered in PMA.control training session " + str(pmacontrolTrainingSessionID))
+	url = pma._pma_join(pmacontrolURL, "api/Sessions/") + str(pmacontrolTrainingSessionID) + "/AddParticipant?SessionID=" + pmacoreSessionID
+	data = { "UserName": participantUsername, "Role": pmacontrolRole, "InteractionMode": pmacontrolInteractionMode}   # default interaction mode = Locked
 	data = parse.urlencode(data).encode()
 	if (pma._pma_debug == True):
 		print("Posting to", url)
+		print("   with payload", data)
 	req =  request.Request(url=url, data=data) # this makes the method "POST"
 	resp = request.urlopen(req)
 	pma._pma_clear_url_cache()
 	return resp
 
-def register_participant_for_project(pmacontrolURL, participantUsername, pmacontrolProjectID, pmacontrolRole, pmacoreSessionID):
+def register_participant_for_project(pmacontrolURL, participantUsername, pmacontrolProjectID, pmacontrolRole, pmacoreSessionID, pmacontrolInteractionMode = pma_interaction_mode_locked):
 	"""
 	Registers a participant for all sessions in a given project, assigning a specific role
 	"""
-	registered_sessions = []
-	for session in get_training_sessions(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
-		register_participant_for_training_session(pmacontrolURL, participantUsername, session, pmacontrolRole, pmacoreSessionID)
-		registered_sessions.append(session)
-	if (pma._pma_debug == True):
-		print(participantUsername, "is now registered in", registered_sessions)
-	return registered_sessions
-
-def set_participant_interactionmode(pmacontrolURL, participantUsername, pmacontrolSessionID, pmacontrolCaseCollectionID, pmacontrolInteractionMode, pmacoreSessionID):
-	"""
-	Assign an interaction mode to a particpant for a given Case Collection within a trainingsession
-	"""
-	if not is_participant_in_training_session(pmacontrolURL, participantUsername, pmacontrolSessionID, pmacoreSessionID):
-		raise NameError ("PMA.core user " + participantUsername + " is NOT registered in PMA.control training session " + str(pmacontrolSessionID))
-	url = pma._pma_join(pmacontrolURL, "api/Sessions/") + str(pmacontrolSessionID) + "/InteractionMode?SessionID=" + pmacoreSessionID
-	data = { "UserName": participantUsername, "CaseCollectionId": pmacontrolCaseCollectionID, "InteractionMode": pmacontrolInteractionMode }   
+	url = pma._pma_join(pmacontrolURL, "api/Projects/") + str(pmacontrolProjectID) + "/AddParticipant?SessionID=" + pmacoreSessionID
+	data = { "UserName": participantUsername, "Role": pmacontrolRole, "InteractionMode": pmacontrolInteractionMode }   # default interaction mode = Locked
 	data = parse.urlencode(data).encode()
 	if (pma._pma_debug == True):
 		print("Posting to", url)
+		print("   with payload", data)
 	req =  request.Request(url=url, data=data) # this makes the method "POST"
 	resp = request.urlopen(req)
 	pma._pma_clear_url_cache()
 	return resp
 
+def _pma_get_case_collection_training_session_id(pmacontrolURL, pmacontrolTrainingSessionID, pmacontrolCaseCollectionID, pmacoreSessionID):
+	full_training_sessions = _pma_get_training_sessions(pmacontrolURL, pmacoreSessionID)
+	new_training_session_dict = {}
+	for sess in full_training_sessions:
+		if sess["Id"] == pmacontrolTrainingSessionID:
+			for coll in sess["CaseCollections"]:
+				if coll["CaseCollectionId"] == pmacontrolCaseCollectionID:
+					return coll["Id"]
+	return None
+
+def set_participant_interactionmode(pmacontrolURL, participantUsername, pmacontrolTrainingSessionID, pmacontrolCaseCollectionID, pmacontrolInteractionMode, pmacoreSessionID):
+	"""
+	Assign an interaction mode to a particpant for a given Case Collection within a trainingsession
+	"""
+	if not is_participant_in_training_session(pmacontrolURL, participantUsername, pmacontrolTrainingSessionID, pmacoreSessionID):
+		raise NameError ("PMA.core user " + participantUsername + " is NOT registered in PMA.control training session " + str(pmacontrolTrainingSessionID))
+	url = pma._pma_join(pmacontrolURL, "api/Sessions/") + str(pmacontrolTrainingSessionID) + "/InteractionMode?SessionID=" + pmacoreSessionID
+	data = { "UserName": participantUsername, "CaseCollectionId": pmacontrolCaseCollectionID, "InteractionMode": pmacontrolInteractionMode }   
+	#pmacontrolTrainingSessionCaseCollectionID = _pma_get_case_collection_training_session_id(pmacontrolURL, pmacontrolTrainingSessionID, pmacontrolCaseCollectionID, pmacoreSessionID)
+	#data = { "UserName": participantUsername, "CaseCollectionId": pmacontrolTrainingSessionCaseCollectionID, "InteractionMode": pmacontrolInteractionMode }   
+	data = parse.urlencode(data).encode()
+	if (pma._pma_debug == True):
+		print("Posting to", url)
+		print("   with payload", data)
+	req =  request.Request(url=url, data=data) # this makes the method "POST"
+	try:
+		resp = request.urlopen(req)
+	except urllib.error.HTTPError as e:
+		if (pma._pma_debug == True):
+			print ("HTTP ERROR")
+			print (e.__dict__)
+		return None
+	except urllib.error.URLError as e:
+		if (pma._pma_debug == True):
+			print ("URL ERROR")
+			print (e.__dict__)
+		return None
+	pma._pma_clear_url_cache()
+	return resp
+
 def get_training_session_titles(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
 	"""
 	Retrieve sessions (possibly filtered by project ID), titles only
@@ -215,14 +244,28 @@ def get_training_session_titles_dict(pmacontrolURL, pmacontrolProjectID, pmacore
 
 	return dct
 
-def get_training_session(pmacontrolURL, pmacontrolSessionID, pmacoreSessionID):
+def get_training_sessions(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
 	"""
-	Return the first (training) session with ID = pmacontrolSessionID
+	Retrieve (training) sessions (possibly filtered by project ID), return a dictionary of session IDs and titles
+	"""
+	dct = {}
+	all = _pma_get_training_sessions(pmacontrolURL, pmacoreSessionID)
+	for sess in all:
+		if pmacontrolProjectID == None:
+			dct[sess["Id"]] = _pma_format_training_session_properly(sess)
+		elif pmacontrolProjectID == sess["ProjectId"]:
+			dct[sess["Id"]] = _pma_format_training_session_properly(sess)
+
+	return dct
+
+def get_training_session(pmacontrolURL, pmacontrolTrainingSessionID, pmacoreSessionID):
+	"""
+	Return the first (training) session with ID = pmacontrolTrainingSessionID
 	"""
 	all = _pma_get_training_sessions(pmacontrolURL, pmacoreSessionID)
 
 	for el in all:
-		if pmacontrolSessionID ==  el['Id']:
+		if pmacontrolTrainingSessionID ==  el['Id']:
 			# summarize session-related information so that it makes sense
 			return _pma_format_training_session_properly(el)
 
@@ -251,12 +294,24 @@ def _pma_get_case_collections(pmacontrolURL, pmacoreSessionID):
 	url = pma._pma_join(pmacontrolURL, "api/CaseCollections?sessionID=" + pma._pma_q(pmacoreSessionID))
 	try:
 		headers = {'Accept': 'application/json'}
-		# r = requests.get(url, headers=headers)
 		r = pma._pma_http_get(url, headers)
 		return r.json()
 	except Exception as e:
-		return None		
+		return None
 
+
+def get_case_collections(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
+	"""
+	Retrieve case collection details that belong to a specific project
+	"""
+	colls = {}
+	all_colls = _pma_get_case_collections(pmacontrolURL, pmacoreSessionID)
+	for coll in all_colls:
+		if (pmacontrolProjectID == coll["ProjectId"]) and not (coll["Id"] in colls.keys()):
+			colls[coll["Id"]] = coll
+
+	return colls
+	
 def get_case_collection_titles(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
 	"""
 	Retrieve case collections (possibly filtered by project ID), titles only
@@ -264,8 +319,8 @@ def get_case_collection_titles(pmacontrolURL, pmacontrolProjectID, pmacoreSessio
 	try:
 		return list(get_case_collection_titles_dict(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID).values())
 	except Exception as e:
-		return None		
-	
+		return None
+
 def get_case_collection_titles_dict(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
 	"""
 	Retrieve case collections (possibly filtered by project ID), return a dictionary of case collection IDs and titles
@@ -318,24 +373,42 @@ def _pma_format_project_embedded_training_sessions_properly(original_project_ses
 		dct[prj_sess["Id"]] = prj_sess["Title"]
 
 	return dct
-	
+
 def _pma_get_projects(pmacontrolURL, pmacoreSessionID):
 	"""
 	Retrieve all projects and their data in PMA.control 
 	(RAW JSON data; not suited for human consumption)
 	"""
 	global _pma_projects_json
-	
+
 	url = pma._pma_join(pmacontrolURL, "api/Projects?sessionID=" + pma._pma_q(pmacoreSessionID))
-	print (url)
 	try:
 		headers = {'Accept': 'application/json'}
-		# r = requests.get(url, headers=headers)
-		r = pma._pma_http_get(url, headers)		
+		r = pma._pma_http_get(url, headers)
 		return r.json()
 	except Exception as e:
 		return None		
 
+def get_projects(pmacontrolURL, pmacoreSessionID):
+	"""
+	Retrieve project details for all projects
+	"""
+	all_projects = _pma_get_projects(pmacontrolURL, pmacoreSessionID)
+	projects = {}
+	for prj in all_projects:
+		# summary session-related information so that it makes sense
+		prj['Sessions'] = _pma_format_project_embedded_training_sessions_properly(prj['Sessions'])
+		
+		# now integrate case collection information
+		colls = _pma_get_case_collections(pmacontrolURL, pmacoreSessionID)
+		prj['CaseCollections'] = {}
+		for col in colls:
+			if col['ProjectId'] == prj['Id']:
+				prj['CaseCollections'][col['Id']] = col['Title']
+		projects[prj['Id']] = prj
+		
+	return projects
+
 def get_project_titles(pmacontrolURL, pmacoreSessionID):
 	"""
 	Retrieve projects, return ONLY the titles
@@ -343,7 +416,7 @@ def get_project_titles(pmacontrolURL, pmacoreSessionID):
 	try:
 		return list(get_project_titles_dict(pmacontrolURL, pmacoreSessionID).values())
 	except Exception as e:
-		return None		
+		return None
 
 def get_project_titles_dict(pmacontrolURL, pmacoreSessionID):
 	"""
@@ -356,8 +429,8 @@ def get_project_titles_dict(pmacontrolURL, pmacoreSessionID):
 			dct[prj['Id']] = prj['Title']
 	except Exception as e:
 		print(e)
-		return None		
-		
+		return None
+
 	return dct
 
 def get_project(pmacontrolURL, pmacontrolProjectID, pmacoreSessionID):
@@ -392,7 +465,7 @@ def get_project_by_case_id(pmacontrolURL, pmacontrolCaseID, pmacoreSessionID):
 				return get_project(pmacontrolURL, coll["ProjectId"], pmacoreSessionID)
 
 	return None
-	
+
 def get_project_by_case_collection_id(pmacontrolURL, pmacontrolCaseCollectionID, pmacoreSessionID):
 	"""
 	Retrieve case collection based on the case ID
@@ -421,7 +494,7 @@ def search_project(pmacontrolURL, titleSubstring, pmacoreSessionID):
 			for col in colls:
 				if col['ProjectId'] == prj['Id']:
 					prj['CaseCollections'][col['Id']] = col['Title']
-					
+
 			return prj
 
 	return None
\ No newline at end of file
